<?php
namespace Bunyad\Blocks\Base;

use Bunyad;
use WP_Query;

/**
 * Base Blocks Class for Loops type blocks - should be extended for other blocks
 * 
 * Note: This is not an abstract class and may be used standalone. More of template method.
 */
class LoopBlock extends Block
{
	/**
	 * @var string Block type
	 */
	public $type = 'loop';

	/**
	 * @var boolean Is the block processed yet
	 */
	public $processed = false;

	/**
	 * @var array Internal data, extended by data retrieved from \Query::setup()
	 */
	public $data = [
		'unique_id' => null
	];

	/**
	 * @var \WP_Query
	 */
	public $query;

	/**
	 * @param array $props
	 */
	public function __construct($props = [])
	{
		parent::__construct($props);
		
		// Resolve aliases
		$this->props = $this->resolve_aliases($this->props);

		// Setup enqueues if any
		add_action('wp_enqueue_scripts', [$this, 'register_assets']);
	}

	/**
	 * Default properties for the block.
	 * 
	 * @param string $type Type of props to return.
	 * @return array
	 */
	public static function get_default_props($type = null)
	{	
		$query_props = [
			'posts'        => 4,
			'offset'       => '',

			// Main category
			'cat'          => '',

			// Tag slugs - separated by commas. Not in 'terms' to allow limit by tags AND cats.
			'tags'         => [],

			// Categories, or custom taxonomies' term ids, or author ids.
			'terms'        => [],

			// Limit to a specific custom taxonomy
			'taxonomy'     => '',

			// Custom taxonomy IDs to use
			'tax_ids'      => [],
			'sort_order'   => '',
			'sort_by'      => '',
			'post_formats' => [],

			// Multiple supported only for legacy compat. Recommended: Single value.
			'post_type'    => '',

			// Specific post IDs
			'post_ids'     => [],

			'pagination'      => '',
			'pagination_type' => '',
		];

		// Only return one type of props if specified.
		if ($type == 'query') {
			return $query_props;
		}

		// Config props (not for sc)
		$props = [

			// Use global $wp_query or legacy $bunyad_loop, ignores all query props
			'use_query_props' => false,

			// Forces to use the specified query
			'query'           => null,

			// Whether is it called via a shortcode / builder - adds .block-sc to wrap
			'is_sc_call'      => false,

			// Style Variation for the block.
			'style'         => '',

			// Many blocks support columns setting.
			'columns'        => '',
			'columns_main'   => '',
			'columns_medium' => '',
			'columns_small'  => '',

			'heading'       => '',
			'heading_type'  => '',
			'heading_link'  => '',

			// Values: category, tag, or taxonomy - if empty, defaults to category 
			'filters'       => false,
			'filters_terms' => [],

			// When using custom taxonomy
			'filters_tax'   => '',

			// Empty or preload enables filters preloading
			'filters_load'  => 'preload',

			// Loop specific props
			'excerpts'        => true,
			'excerpt_length'  => 15,
			'cat_labels'      => true,
			'read_more'       => '',

			// When empty, automatically inferred based on columns.
			'show_media'         => true,
			'force_image'        => '',
			'media_ratio'        => '',
			'media_ratio_custom' => '',

			// Only for some blocks: list, small.
			'media_width'        => '',

			// Define a parent container to get relative width from.
			'container_width' => '',

			'separators'      => true,

			// Margin below block
			'space_below'     => '',

			'meta_items_default' => true,
			'meta_above'         => [],
			'meta_below'         => [],

			// These meta items should not get a default value as global settings are important.
			'meta_align'         => '',
			'meta_cat_style'     => 'text',
			'meta_author_img'    => false,

			// Aliases
			'filters_tags' => ['alias' => 'filters_terms'],

			// Legacy Aliases (DEPRECATED)
			'link'         => ['alias' => 'heading_link'],
			'post_format'  => ['alias' => 'post_formats'],
			'title'        => ['alias' => 'heading'],
			'cats'         => ['alias' => 'terms'],
		];

		$props = $props + $query_props;

		/**
		 * Setup some global defaults if options available.
		 */
		if (Bunyad::options()) {
			$props = array_replace($props, [
				'meta_above' => Bunyad::options()->get('post_meta_above'),
				'meta_below' => Bunyad::options()->get('post_meta_below'),
			]);
		}

		return $props;
	}

	/**
	 * @inheritDoc 
	 */
	public function init()
	{
		// Setup internal props. These are not set from external call.
		$this->props += [
			'image'        => '',
			'image_props'  => [],
			'class_grid'   => ['grid'],
		];
	}

	/**
	 * @inheritDoc
	 */
	public function setup_props($props) 
	{
		$props = parent::setup_props($props);

		if (!empty($props['is_sc_call'])) {
			// $props = $this->map_external_props($props);
		}

		if (isset($props['columns_main'])) {
			$props['columns'] = $props['columns_main'];
		}

		return $props;
	}

	/**
	 * Set the block identifier
	 * 
	 * @return $this
	 */
	public function set_id($id) 
	{
		$this->id = $id;
		return $this;
	}

	/**
	 * Set a single property - sets on $this->props[] array
	 * 
	 * @return $this
	 */
	public function set($key, $value) 
	{
		$this->props[$key] = $value;
		return $this;
	}

	/**
	 * Get all props.
	 * 
	 * @see $this::get_default_props()
	 * @return array
	 */
	public function get_props()
	{
		return $this->props;
	}

	/**
	 * Setup aliases for provided props.
	 */
	public function resolve_aliases($props)
	{
		$default = $this->get_default_props();

		foreach ($default as $key => $prop) {

			// Is not an alias? Skip
			if (!is_array($prop) OR !array_key_exists('alias', $prop)) {
				continue;
			}
			
			// Alias used here
			if (array_key_exists($key, $props)) {

				// A stray config prop, remove it
				if (!empty($props[$key]['alias'])) {

					unset($props[$key]);
					continue;
				}

				// Example: 'terms' set from $props['cats']
				$props[ $prop['alias'] ] = $props[$key];
			}
		}

		return $props;
	}

	/**
	 * Any assets to register for this block.
	 */
	public function register_assets() {}

	/**
	 * Process and setup query.
	 * 
	 * @uses \Bunyad\Blocks\Base\Query
	 */
	public function process()
	{
		$this->create_unique_id();

		// Using a custom or global query
		if (!$this->props['use_query_props']) {

			if ($this->props['query'] instanceof WP_Query) {

				$this->query = $this->props['query'];

				/**
				 * Change post_count to limit to post number specified
				 */
				// $query = $this->query;
				
				// if ($query->post_count > $this->props['posts']) {
				// 	$query->post_count = $this->props['posts'];
				// }

				// $this->query = $query;
			}
			else {
				// Note: Block data such as title, link, main_term is not available
				$this->query = !empty($GLOBALS['bunyad_loop']) ? $GLOBALS['bunyad_loop'] : $GLOBALS['wp_query'];
			}
		}
		else {

			// Setup the block query
			$query = new Query($this);
			$this->data = array_merge($this->data, $query->setup());

			$this->query = $this->data['query'];
			unset($this->data['query']);
		}

		// Flag to mark processed
		$this->processed = true;
	}

	/**
	 * Render the partial view for this loop.
	 * 
	 * @uses \Bunyad_Core::partial()
	 */
	public function render($options = array())
	{
		$options = wp_parse_args($options, array(
			'block_markup' => true
		));

		$this->_pre_render();
		$this->setup_media_ratio();
		$this->infer_image_sizes();

		if (!$this->processed) {
			$this->process();
		}

		// Render with or without block markup
		($options['block_markup'] ? $this->render_block_markup() : $this->render_view());
	}

	/**
	 * Checks to perform and settings to do prior to render
	 */
	public function _pre_render() {}

	/**
	 * Render the view file for this block - usually a loop
	 */
	public function render_view()
	{
		Bunyad::core()->partial(
			'/blocks/loops/' . $this->id . '/html/' . $this->id,
			array(
				'block' => $this,
				'query' => $this->query
			)
		);
	}

	/**
	 * Output common block markup and render the specific view file.
	 * 
	 * Note: Do not call directly, use render() instead.
	 * 
	 * @uses self::render_view()
	 */
	public function render_block_markup()
	{
		$margins = '';
		if (!empty($this->props['space_below'])) {
			$margins = 'mb-' . $this->props['space_below'];
		}

		?>
		<section <?php 
			Bunyad::markup()->attribs(
				$this->id .'-block', 
				array(
					'class' => array(
						'block-wrap', 
						'block-' . $this->id, 
						($this->props['is_sc_call'] ? 'block-sc' : ''),
						$margins
					), 
					'data-id' => $this->data['unique_id']
				)
			); ?>>

			<?php $this->the_heading(); ?>
	
			<div class="block-content">
				<?php $this->render_view(); ?>
			</div>

		</section>
		<?php
	}

	/**
	 * Created unique ID for the block
	 */
	protected function create_unique_id()
	{
		Bunyad::registry()->block_count++;
		$this->data['unique_id'] = Bunyad::registry()->block_count;
	}

	/**
	 * Output block heading markup
	 */
	public function the_heading()
	{
		// No heading processed? It's important the process() run first
		if (empty($this->data['heading'])) {
			return;
		}
		
		$props = $this->props;
		
		// Heading disabled?
		if ($props['heading_type'] == 'none') {
			return;
		}
		
		// Get display filters to show in heading
		$filters = $this->get_the_filters();

		// Set heading with link if possible
		if (!empty($this->data['heading_link'])) {
			$heading = '<a href="' . esc_url($this->data['heading_link']) .'">' . esc_html($this->data['heading']) . '</a>';
		}
		
		ob_start();

		?>
					
			<?php
			/**
			 * Heading types: empty/default, block-filter or block
			 */
			if (!$props['heading_type'] || in_array($props['heading_type'], array('default', 'block-filter', 'block'))): 
					
					$classes = array('section-head prominent heading');

					if (is_object($this->data['term'])) { 
						$classes[] = 'cat-text-' . $this->data['term']->term_id;
					}
					
					$heading = preg_replace('/(\*|_)(.*?)\1/', '<span class="color">\2</span>', $heading);

			?>
				
				<div <?php Bunyad::markup()->attribs('block-heading-wrap', array('class' => $classes)); ?>>
					<h3 class="heading"><?php echo $heading; ?></h3>
										
					<?php echo $filters; ?>

				</div>
				
			<?php 
			/**
			 * Heading type: block-alt
			 */
			elseif ($heading_type == 'block-alt'): 

					$classes = array('block-head');
					
					if (is_object($this->data['term'])) { 
						$classes[] = 'cat-text-' . $this->data['term']->term_id;
					}
			?>

				<div <?php Bunyad::markup()->attribs('block-heading-wrap', array('class' => $classes)); ?>>
					<h3 class="heading"><?php echo $heading; ?></h3>
										
					<?php echo $filters; ?>

				</div>			

			<?php 
			/**
			 * Heading type: Page
			 */
			else:
			?>

				<h2 class="main-heading prominent"><?php echo $heading; // Escaped earlier when added ?></h2>

			<?php endif; ?>

		<?php

		echo apply_filters('bunyad/blocks/loops/the_heading', ob_get_clean());

	}

	/**
	 * Get the filters output
	 */
	public function get_the_filters()
	{
		$display_filters = $this->data['display_filters'];

		ob_start();
		
		?>
		
		<ul class="subcats filters">
			<li><a href="#" data-id="0" class="active"><?php esc_html_e('All', 'bunyad'); ?></a></li>
			<?php echo join("\n", $display_filters); ?>
		</ul>
		
		<?php 

		return apply_filters('bunyad/blocks/loops/the_filters', ob_get_clean());
	}

	/**
	 * Load a loop post class
	 * 
	 * Example: Load the loop class from inc/loop-posts/grid.php if it exists 
	 * or fallback to inc/loop-posts/base.php
	 * 
	 * @uses \Bunyad\Blocks\Helpers::load_post()
	 * 
	 * @param string $id     Loop post id
	 * @param array  $props  Props to set for loop post
	 *
	 * @return \Bunyad\Blocks\LoopPosts\BasePost
	 */
	public function loop_post($id, $props = array()) 
	{
		$props = array_merge($this->get_props(), $props);

		// Load post
		$post = Bunyad::blocks()->load_post($id, $props);
		$post->block = $this;

		return $post;
	}

	/**
	 * Get relative width for current block, based on parent column width in 
	 * relation to the whole page.
	 * 
	 * @return float Column width in percent number, example 66
	 */
	public function get_relative_width()
	{
		// Container defined in props, force it
		if (!empty($this->props['container_width'])) {
			return floatval($this->props['container_width']);
		}

		// Set current column width weight (width/100) - used to determine image sizing 
		$col_relative_width = 1;
		if (isset(Bunyad::registry()->layout['col_relative_width'])) {
			$col_relative_width = Bunyad::registry()->layout['col_relative_width'];
		}
	
		// Adjust relative width if there's a sidebar
		if (Bunyad::core()->get_sidebar() != 'none') {
			$col_relative_width = ($col_relative_width * (8/12));
		}

		return $col_relative_width * 100;
	}

	/**
	 * Set columns size based on provided columns.
	 *
	 * @param array $args Configs for columns of devices.
	 * @return void
	 */
	protected function setup_columns($args = []) 
	{
		if (empty($this->props['columns'])) {
			$this->props['columns'] = 1;
		}

		// Add the grid columns class.
		$this->props['class_grid'][] = 'grid-' . $this->props['columns'];

		$col_types = [
			'md' => 'medium', 
			'sm' => 'small', 
			'xs' => 'xsmall'
		];

		/**
		 * Add responsive column classes based on props of type columns_medium etc.
		 * OR, override via args provided.
		 */
		foreach ($col_types as $key => $columns) {

			$cols = null;
			if (!empty($this->props[ 'columns_' . $columns ])) {
				$cols = $this->props[ 'columns_' . $columns ];
			}
			else if (!empty($args[ $columns ])) {
				$cols = $args[ $columns ];
			}

			if ($cols) {
				array_push(
					$this->props['class_grid'], 
					"{$key}:grid-{$cols}"
				);
			}
		}

		 // 1/3 * 12 to get col-4
		 $column = (1 / absint($this->props['columns']) * 12);
		 $this->props['col_class'] = 'col-' . str_replace('.', '-', $column);
	}

	/**
	 * Setup media ratio if provided.
	 */
	public function setup_media_ratio()
	{
		if (empty($this->props['media_ratio'])) {
			return;
		}

		$ratio = $this->props['media_ratio'];
		if ($ratio === 'custom' && !empty($this->props['media_ratio_custom'])) {
			$ratio = $this->props['media_ratio_custom'];
		}

		$this->props['media_ratio'] = $ratio;
	}

	/**
	 * Decide the image to use for this block
	 */
	public function infer_image_sizes()
	{
		// If an image is forced, don't bother setting it
		if (!empty($this->props['image'])) {
			return;
		}

		$this->props['image'] = 'bunyad-thumb';
	}
	

	/**
	 * Render pagination for current block
	 * 
	 * @uses \Bunyad_Core::partial()
	 * @param array $props
	 */
	public function the_pagination($props = array())
	{
		$props = array_merge(
			array(
				'query'           => $this->query,
				'pagination'      => $this->props['pagination'],
				'pagination_type' => $this->props['pagination_type']
			),
			$props
		);

		Bunyad::core()->partial('pagination', $props);
	}

}